const express = require("express");
const app = express();
const {log} = require("./logger");
const axios = require("axios");


let IP_ADDRESSES = [];
const os = require('os');
const networkInterfaces = os.networkInterfaces();

// Iterate through network interfaces
for (const name in networkInterfaces) {
  const interfaces = networkInterfaces[name];
  for (const iface of interfaces) {
    // Ensure it's an IPv4 address and not internal
    if (iface.family === 'IPv4' && !iface.internal) {
      IP_ADDRESSES.push(iface.address);
      // You might want to store or return this address
    }
  }
}


// MIDDLEWARE
app.use(express.static('public'));
app.set('view engine', 'ejs');
app.use(express.json());
app.use((req, res, next) => {
  log.info(`${req.method} ${req.url}`);
  next();
});

// ROUTES
app.get('/', (req, res) => {
  res.render("home", {
    title:"Home Page",
    IP_ADDRESSES: JSON.stringify(IP_ADDRESSES)
  })
});

app.get('/load-balancing', (req, res) => {
  res.render("load-balancing", {
    title:"Load Balancing"
  })
});

app.get('/instance-metadata', (req, res) => {
  res.render("instance-metadata", {
    title:"Instance Metadata"
  })
});

app.get("/instance-metadata/get-metadata", async (req, res, next) => {
  //curl -X PUT "http://169.254.169.254/latest/api/token" -H "X-aws-ec2-metadata-token-ttl-seconds: 21600"
  //curl -H "X-aws-ec2-metadata-token: $TOKEN" http://169.254.169.254/latest/meta-data/
  //curl -H "X-aws-ec2-metadata-token: $TOKEN" http://169.254.169.254/latest/meta-data/hostname
  
  const baseURL = "http://169.254.169.254/"
  try{
    const tokenResponse = await axios.put(baseURL + "latest/api/token", null, {headers:{"X-aws-ec2-metadata-token-ttl-seconds": 360}});
    const token = tokenResponse.data;
    log.info("Instance Metadata Token:" + token);
    const metadata = await axios.get(baseURL + "latest/api/meta-data", {headers:{"X-aws-ec2-metadata-token": token}})
    res.json(metadata);
  }catch(err){
    next(err);
    //console.log("Error fetching instance metadata:", err);
    //res.status(500).send("Error fetching instance metadata:" + JSON.stringify(err))
  }

});


let unhealthyUntil = 0;
app.get('/health', (req, res) => {
  if(unhealthyUntil > new Date().getTime()){
    res.status(500).send('UNHEALTHY');
  }else{
    res.send("healthy...");
  }
  
});

app.get('/make-unhealthy', (req, res) => {
  unhealthyUntil = req.query.until;
  //console.log("duration", req.query.until);
  res.send("Unhealthy until: " + new Date(req.query.until).toDateString());
});

// ERROR HANDLER
app.use((err, req, res, next) => {

  const requestedUrl = req.protocol + '://' + req.get('host') + req.originalUrl;
  const statusCode = err.statusCode ||  500;
    
  log.error(err.message, {url: requestedUrl, method: req.method, statusCode: statusCode, stack: err.stack});
  res.status(statusCode).json({message: err.message, stacktrace: err.stack});

});

// START THE SERVER
const port = process.env.PORT;
const server = app.listen(port, () => {
  console.log("Waiting for requests on port %s", port);
});